==> kleopatra-16.08.1.tar.xz: kleopatra-16.08.1/src/utils/validation.cpp <==
/* -*- mode: c++; c-basic-offset:4 -*-
    utils/validation.cpp

    This file is part of Kleopatra, the KDE keymanager
    Copyright (c) 2008 Klarälvdalens Datakonsult AB

    Kleopatra is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kleopatra is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    In addition, as a special exception, the copyright holders give
    permission to link the code of this program with any edition of
    the Qt library by Trolltech AS, Norway (or with modified versions
    of Qt that use the same license as Qt), and distribute linked
    combinations including the two.  You must obey the GNU General
    Public License in all respects for all of the code used other than
    Qt.  If you modify this file, you may extend this exception to
    your version of the file, but you are not obligated to do so.  If
    you do not wish to do so, delete this exception statement from
    your version.
*/

#include <config-kleopatra.h>

#include "validation.h"

#include <utils/multivalidator.h>

#include "kleopatra_debug.h"

#include <QRegExp>
#include <QUrl>

#include <cassert>

using namespace Kleo;

static const QString email_rx = QStringLiteral("[a-zA-Z0-9!#$%&'*+/=?^_`{|}~-]+(?:\\.[a-zA-Z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?\\.)+[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?");
// these are modeled after gnupg/g10/keygen.c:ask_user_id:
static const QString name_rx = QStringLiteral("[^0-9<>][^<>@]{4,}");
static const QString comment_rx = QStringLiteral("[^()]*");

namespace
{

class EMailValidator : public QValidator
{
    QRegExp rx;
public:
    explicit EMailValidator(QObject *parent = Q_NULLPTR) : QValidator(parent), rx(QRegExp(email_rx)) {}

    void fixup(QString &) const Q_DECL_OVERRIDE {}

    State validate(QString &str, int &pos) const Q_DECL_OVERRIDE
    {
        const int atIdx = str.lastIndexOf(QLatin1Char('@'));
        if (atIdx < 0 || str.endsWith(QLatin1Char('@'))) {
            return regexValidate(str, pos);
        }

        // toAce/fromAce doesn't like intermediate domain names,
        // so we fix them up with something innocuous to help it
        // along, and which we strip again afterwards

        QString domain = str.mid(atIdx + 1).toLower();
        const int dotIndex = domain.lastIndexOf(QLatin1Char('.'));
        const bool needsOrgAdded = domain.endsWith(QLatin1Char('.'));
        // during typing, the domain might end with '-', which is okay
        // yeah, foo.s also disrupts fromAce, during typing this is okay
        const bool needsDotOrgAdded = !needsOrgAdded && (dotIndex < 0 || dotIndex == domain.size() - 2 || domain.endsWith(QLatin1Char('-')));
        if (needsOrgAdded) {
            domain += QLatin1String("org");
        }
        if (needsDotOrgAdded) {
            domain += QLatin1String("tmp.org");
        }
        const QByteArray domainEncoded = QUrl::toAce(domain);
        const QString domainRestored = QUrl::fromAce(domainEncoded);
        QString encoded = str.left(atIdx) + QLatin1Char('@') + QString::fromLatin1(domainEncoded);
        if (needsDotOrgAdded) {
            assert(encoded.endsWith(QStringLiteral("tmp.org")));
            encoded.chop(7);
        }
        if (needsOrgAdded) {
            assert(encoded.endsWith(QStringLiteral(".org")));
            encoded.chop(3);   // '.' was part of domain before
        }
        qCDebug(KLEOPATRA_LOG) << "\n str           :" << str
                               << "\n domain        :" << domain
                               << "\n domainEncoded :" << domainEncoded
                               << "\n domainRestored:" << domainRestored
                               << "\n encoded       :" << encoded;
        if (domain != domainRestored) {
            return Invalid;
        }

        // there's no difference between 'encoded' and 'str' at
        // least up to and including 'atIdx', and we need the
        // position for the fixed Intermediate state in
        // regexValidate (e.g. adding a . after marc in
        // marc@kdab.com, intending to eventually arrive at
        // marc.mutz@kdab.com)
        int adjustedPos = pos <= atIdx ? pos : encoded.size();
        return regexValidate(encoded, adjustedPos);
    }

private:
    State regexValidate(QString &input, int &pos) const
    {
        // fixed version of QRegExpValidator::validate():
        if (rx.exactMatch(input)) {
            return Acceptable;
        } else {
            if (const_cast<QRegExp &>(rx).matchedLength() >= /*input.size()*/pos) {
                return Intermediate;
            } else {
                pos = input.size();
                return Invalid;
            }
        }
    }

};

}

QValidator *Validation::email(QObject *parent)
{
    return new EMailValidator(parent);
}

QValidator *Validation::email(const QRegExp &addRX, QObject *parent)
{
    return new MultiValidator(email(), new QRegExpValidator(addRX, 0), parent);
}

QValidator *Validation::pgpName(QObject *parent)
{
    return new QRegExpValidator(QRegExp(name_rx), parent);
}

QValidator *Validation::pgpName(const QRegExp &addRX, QObject *parent)
{
    return new MultiValidator(pgpName(), new QRegExpValidator(addRX, 0), parent);
}

QValidator *Validation::pgpComment(QObject *parent)
{
    return new QRegExpValidator(QRegExp(comment_rx), parent);
}

QValidator *Validation::pgpComment(const QRegExp &addRX, QObject *parent)
{
    return new MultiValidator(pgpComment(), new QRegExpValidator(addRX, 0), parent);
}

==> kleopatra-17.08.0.tar.xz: kleopatra-17.08.0/src/utils/validation.cpp <==
/* -*- mode: c++; c-basic-offset:4 -*-
    utils/validation.cpp

    This file is part of Kleopatra, the KDE keymanager
    Copyright (c) 2008 Klarälvdalens Datakonsult AB

    Kleopatra is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kleopatra is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    In addition, as a special exception, the copyright holders give
    permission to link the code of this program with any edition of
    the Qt library by Trolltech AS, Norway (or with modified versions
    of Qt that use the same license as Qt), and distribute linked
    combinations including the two.  You must obey the GNU General
    Public License in all respects for all of the code used other than
    Qt.  If you modify this file, you may extend this exception to
    your version of the file, but you are not obligated to do so.  If
    you do not wish to do so, delete this exception statement from
    your version.
*/

#include <config-kleopatra.h>

#include "validation.h"

#include <utils/multivalidator.h>

#include <KEmailAddress>

#include "kleopatra_debug.h"

#include <QRegExp>
#include <QUrl>

#include <cassert>

using namespace Kleo;

// these are modeled after gnupg/g10/keygen.c:ask_user_id:
static const QString name_rx = QStringLiteral("[^0-9<>][^<>@]{4,}");
static const QString comment_rx = QStringLiteral("[^()]*");

namespace
{

class EMailValidator : public QValidator
{
public:
    explicit EMailValidator(QObject *parent = nullptr) : QValidator(parent) {}

    State validate(QString &str, int &pos) const override
    {
        Q_UNUSED(pos);
        if (KEmailAddress::isValidSimpleAddress(str)) {
            return Acceptable;
        }
        return Intermediate;
    }
};

}

QValidator *Validation::email(QObject *parent)
{
    return new EMailValidator(parent);
}

QValidator *Validation::email(const QRegExp &addRX, QObject *parent)
{
    return new MultiValidator(email(), new QRegExpValidator(addRX, nullptr), parent);
}

QValidator *Validation::pgpName(QObject *parent)
{
    return new QRegExpValidator(QRegExp(name_rx), parent);
}

QValidator *Validation::pgpName(const QRegExp &addRX, QObject *parent)
{
    return new MultiValidator(pgpName(), new QRegExpValidator(addRX, nullptr), parent);
}

QValidator *Validation::pgpComment(QObject *parent)
{
    return new QRegExpValidator(QRegExp(comment_rx), parent);
}

QValidator *Validation::pgpComment(const QRegExp &addRX, QObject *parent)
{
    return new MultiValidator(pgpComment(), new QRegExpValidator(addRX, nullptr), parent);
}
